import multiprocessing
import time
from random import randint
from BrickPi import *   
import subprocess
import os

#To stop the output when playing the alarm 
devnull = open('/dev/null', 'w')

#Plays the alram
def play_alarm(num):
	while(1):
		subprocess.call(["mpg321", "alarm_sound.mp3"],stdout=devnull,stderr=devnull)
	#return
	
#Move the bot forward and turns 90 degree if an obstacle is encountered
#Stops when a touch sensor is pressed on Port 1 or 2
def wait_inp(num):
	ot = time.time()
	while True:
		#Move Motors
		power = 150
		BrickPi.MotorEnable[PORT_A]=1
		BrickPi.MotorEnable[PORT_D]=1
		BrickPi.MotorSpeed[PORT_A] = power  
		BrickPi.MotorSpeed[PORT_D] = power  
		result = BrickPiUpdateValues()  
		
		#Get values from both touch sensors and ultrasonic sensor
		b1=BrickPi.Sensor[PORT_1]
		b2=BrickPi.Sensor[PORT_2]
		udist=BrickPi.Sensor[PORT_4]
		#print udist
		
		#Return to main if any touch sensor is pressed
		if b1 or b2:     
			return
		
		#If the US sensor reads less than 20cm and the new distance value is older by 5secs, rotate 90 degrees
		#(New value checked because while rotating, the US sensor would read less that threshold for a few secs)
		if int(udist)<200 and time.time() - ot > 5:
			ot = time.time()
			power1=[150,150]
			deg=[90,-90]
			port=[PORT_A,PORT_D]
			motorRotateDegree(power1,deg,port,.1)
		time.sleep(.1)     # sleep for 10 ms
		
if __name__ == '__main__':
	while True:
		BrickPiSetup()
		
		#Set up motors, US sensor and touch sensors
		BrickPi.MotorEnable[PORT_A] = 1 #Enable the Motor A
		BrickPi.MotorEnable[PORT_D] = 1 #Enable the Motor B

		BrickPi.SensorType[PORT_4] = TYPE_SENSOR_EV3_US_M0
		BrickPi.SensorType[PORT_1] = TYPE_SENSOR_TOUCH   #Set the type of sensor at PORT_1
		BrickPi.SensorType[PORT_2] = TYPE_SENSOR_TOUCH

		BrickPiSetupSensors()   #Send the properties of sensors to BrickPi
		
		#Set timeout
		BrickPi.Timeout=1000
		print "BrickPiSetTimeout Status :",BrickPiSetTimeout()
		
		#Start two parallel processes
		jobs = []
		
		#Start alarm process
		p1 = multiprocessing.Process(target=play_alarm, args=(0,))
		jobs.append(p1)
		p1.start() 
		
		#Start motor run process
		p3 = multiprocessing.Process(target=wait_inp, args=(1,))
		jobs.append(p3)
		p3.start()

		#The motor run process returns when any button is pressed
		p3.join()
		#Terminate the alarm process when any button press is recieved
		p1.terminate()
		
		#Button 1: Snooze
		#Button 2: Turn Off alarm
		result = BrickPiUpdateValues()
		b1=BrickPi.Sensor[PORT_1]
		b2=BrickPi.Sensor[PORT_2]
		print b1,b2
		
		#Kill the alarm that is already running
		p = subprocess.Popen(['pidof', 'mpg321'], stdout=subprocess.PIPE)
		out, err = p.communicate()
		for line in out.split(" "):
			os.kill(int(line), 9)
		
		#If button 1 is pressed then snooze for 5 minutes
		if b1:
			print "Snooze"
			time.sleep(300)
		
		#If button 2 is pressed then stop the alarm
		if b2:	
			print "exit:"
			break